\section{ Relational categories of $\SL$-fuzzy relations }
In this section we provide suitable instances of certain relational categories using the module %%@
\smalltt{LFuzzyRel}. We then have the bridge to a component-free (i.e., algebraic) treatment of %%@
$\SL$-fuzzy relations.
\begin{code}
module LFuzzyRelCategories (fRelCat,fRelAll,fRelDistrAll,fRelDivAll,fRelDedCat,
                            fRelGogCat,gogCatFromAnti) where

import LFuzzyRel
import LFuzzyRelLattices
import RelAlg
import List (nub)
import Goguen
import Lattice
\end{code}
From the export list one can see the relational categories covered by this module. Furthermore, the %%@
function \smalltt{gogCatFromAnti} is exported. It implements the standard construction of a Goguen %%@
category from given antimorphisms between the set of scalars over an entry lattice $\SL$ and the %%@
lattice of all crisp $\SL$-fuzzy relations over $\SL$.

We start with categories.
\begin{code}
fRelCat :: (Eq e, Eq obj) => Lat e -> [[FRel e obj obj]] -> 
           Cat [obj] (FRel e obj obj) 
fRelCat entryLat allMs = 
   let objs = nub $ foldr (\(rel:rels) -> (src rel:) . (trg rel :)) [] $
              filter (not.null) allMs
       test o1 o2 f = if elem o1 objs && elem o2 objs then f
                      else error "Illegal objects!"
   in Cat { cat_isObj   = flip elem $ objs
           ,cat_isMor   = \o1 o2 z -> test o1 o2 $ 
                             elem z (head $ dropWhile (\(m:ms) -> (src m/=o1) || 
                                                                  (trg m/=o2)   
                                                      ) allMs)
           ,cat_objects = objs
           ,cat_homset  = \o1 o2 -> test o1 o2 $
                             let l = dropWhile (\(m:ms) -> (src m/=o1) || 
                                                           (trg m/=o2)   ) allMs
                             in if null l then [] else head l
           ,cat_source  = src
           ,cat_target  = trg
           ,cat_idmor   = fId entryLat
           ,cat_comp    = fComp }
\end{code}
From the typing we can see the step from the componentwise consideration to the algebraic level. As %%@
we aim at {\em standard} routines, we avoid unnecessarily manifold parametrizations. Obviously, we %%@
only need the entry lattice \smalltt{entryLat} and all morphisms (i.e., all $\SL$-fuzzy relations) %%@
\smalltt{allMs} over \smalltt{entryLat} that shall form the category. Notice that we demand the %%@
morphisms of \smalltt{allMs} to be ordered by source and target. Remember that the functions %%@
\smalltt{getFRelsBy} and \smalltt{getAllFRelsBy} (cf.\ Section \ref{SecLFuzzyRels}), respectively, %%@
can be used to support the determination of \smalltt{allMs}. \\
Finally, we are able to easily compute the list of objects \smalltt{objs} and the remaining functions %%@
of the data structure \smalltt{Cat}.

The rest can be implemented analogously. The different allegories are provided by the next three %%@
functions.
\begin{code}
fRelAll :: (Eq e, Eq obj) => Lat e -> [[FRel e obj obj]] -> 
                             All [obj] (FRel e obj obj) 
fRelAll entryLat allMs = All { all_cat      = fRelCat entryLat allMs
                              ,all_converse = fConv
                              ,all_meet     = fMeet
                              ,all_incl     = fIncl }
               
fRelDistrAll :: (Eq e, Eq obj) => Lat e -> [[FRel e obj obj]] -> 
                                  DistrAll [obj] (FRel e obj obj) 
fRelDistrAll entryLat allMs = 
  DistrAll { distrAll_all     = fRelAll entryLat allMs
            ,distrAll_bottom  = fBot entryLat
            ,distrAll_join    = fJoin
            ,distrAll_atomset = \s t -> lat_atomS $ fRelLat entryLat s t
            ,distrAll_atoms   = \m -> let fLat = fRelLat entryLat (src m) (trg m)
                                      in filter (\y -> lat_lEq fLat y m) $ 
                                                lat_atomS fLat }
                                             
fRelDivAll :: (Eq e, Eq obj) => Lat e -> [[FRel e obj obj]] -> 
              DivAll [obj] (FRel e obj obj) 
fRelDivAll entryLat allMs = 
    DivAll { divAll_distrAll = fRelDistrAll entryLat allMs
            ,divAll_rres = fRRes
            ,divAll_lres = fLRes
            ,divAll_syq  = fSyQ }
\end{code}
The implementations are straightforward. We use the standard functions of the module %%@
\smalltt{LFuzzyRel} to compute the resulting allegory. The same is true for the distributive %%@
allegory. But, here we additionally need the \smalltt{Lattice} and \smalltt{LFuzzyRelLattices} %%@
modules to determine the atom set of the given $\SL$-fuzzy relations between source and target %%@
(\smalltt{distrAll\_atomset}) resp.\ the atoms that can be used to compute a given morphism %%@
(\smalltt{distrAll\_atoms}). For division allegories we need the residual operations. Here we use the %%@
standard implementations \smalltt{fRRes}, \smalltt{fLRes} and \smalltt{fSyQ}, respectively, from the %%@
module \smalltt{LFuzzyRel}.

Finally, Dedekind and Goguen categories can be instantiated through the following functions. They %%@
again only use standard routines of the module \smalltt{LFuzzyRel}.
\begin{code}
fRelDedCat :: (Eq e, Eq obj) => Lat e -> [[FRel e obj obj]] -> 
                                Ded [obj] (FRel e obj obj) 
fRelDedCat entryLat allMs = Ded { ded_divAll = fRelDivAll entryLat allMs

                                 ,ded_top    = fTop entryLat }

fRelGogCat :: (Eq e, Eq obj) => Lat e -> [[FRel e obj obj]] -> 
                                         Gog (Loos e) [obj] (FRel e obj obj) 
fRelGogCat entryLat allMs = Gog { gog_ded    = fRelDedCat entryLat allMs
                                 ,gog_up     = fUp
                                 ,gog_down   = fDown
                                 ,gog_derOp  = fDerOp }
\end{code}
At the end we provide a function to generate a Goguen category $\SG$ from given antimorphisms. For %%@
this construction we normally would have to use antimorhpisms $f : Sc_{\SG}(A)\lora Crisp_{\SG}[A,B] %%@
$ for given objects $A$ and $B$. For $\SL$-fuzzy relations this corresponds to the computation %%@
$R=\bbigsqcup_{u\in\SL} \alpha_{A}^{u};f(\alpha_{A}^{u})$ known from Section \ref{SecFuzzyRel}. This %%@
is rather inefficient. But, we know that the set of scalars of a given source is isomorphic to the %%@
set of elements of the entry lattice. We use this fact to gain efficiency here. Consider the %%@
following lemma.
\begin{lem}
Let $\SL$ be a complete Brouwerian lattice, $R:A\lora B$ be an $\SL$-fuzzy relation and $f: \SL\lora %%@
Crisp_{\SL}[A,B]$ be an antimorphism such that $R=\bbigsqcup_{u\in\SL} \alpha_{A}^{u};f(u)$. Then we %%@
have $R=\bbigsqcup_{u\in\SL}(\Top_{A,A}^{u}\sqcap f(u))$.
\end{lem}
\begin{proof}[{\bf Proof.}]This is immediately seen by the computation
\begin{tabbing}
\qquad \= $R(x,z)$ \= $=$ \= $[\bbigsqcup_{u\in\SL}(\alpha_{A}^{u};f(u))](x,z)$ \\ 
       \>          \> $=$ \> $\bbigvee_{u\in\SL}\bbigvee_{y\in A}(\alpha_{A}^{u}(x,y)\wedge %%@
f(u)(y,z))$ \qquad\qquad \= (definition $;$) \\
       \>          \> $=$ \> $\bbigvee_{u\in\SL}(\alpha_{A}^{u}(x,x)\wedge f(u)(x,z))$
                             \> (definition $\alpha_{A}^{u}$)\\
	   \>          \> $=$ \> $\bbigvee_{u\in\SL}(u\wedge f(u)(x,z))$
                             \> (definition $\alpha_{A}^{u}$)\\
	   \>          \> $=$ \> $\bbigvee_{u\in\SL}(\Top_{AB}^{u}(x,z)\wedge f(u)(x,z))$
                             \> (definition $\Top_{AB}^{u}$)\\
       \>          \> $=$ \> $[\bbigsqcup_{u\in\SL}(\Top_{AB}^{u}\sqcap f(u))](x,z)$.
\end{tabbing}
\end{proof}
Notice that we do not need the antimorphism property of $f$ in the proof. The lemma indeed is true %%@
for arbitrary mappings. We used antimorphisms only for convenience.\\
Hence, we can finally compute a Goguen category given by certain antimorphisms.
\begin{code}
gogCatFromAnti :: (Eq e, Eq obj) => Lat e -> 
                  [[e -> FRel e obj obj]] -> Gog (Loos e) [obj] (FRel e obj obj) 
gogCatFromAnti e ms = fRelGogCat e $ map (computeRels $ lat_jIrredS e) ms 
  where 
    computeRels ir morphs = [foldl1 fJoin [fUpdAllUnBy (lat_inf e irEl) (m irEl)
                                          | irEl<-ir] | m<-morphs]
\end{code}
\emptypage

